@@ -18,7 +18,7 @@ import ncscli.tellInstances as tellInstances
 import startForwarders  # expected to be in the same directory
 
 
-neoloadVersion = '7.10'  # '7.6', '7.7' and '7.10' are currently supported
+neoloadVersion = '7.10.1'  # will be overridden by cmd-line arg
 nlWebWanted = False
 
 class g_:
@@ -26,16 +26,44 @@ class g_:
     interrupted = False
 
 
+def readJLog( inFilePath ):
+    '''read JLog file, return list of decoded objects'''
+    recs = []
+    # read and decode each line as json
+    try:
+        with open( inFilePath, 'rb' ) as inFile:
+            for line in inFile:
+                try:
+                    decoded = json.loads( line )
+                except Exception as exc:
+                    logger.warning( 'exception decoding json (%s) %s', type(exc), exc )
+                recs.append( decoded )
+    except Exception as exc:
+        logger.warning( 'exception reading file (%s) %s', type(exc), exc )
+    return recs
+
+def scriptDirPath():
+    '''returns the absolute path to the directory containing this script'''
+    return os.path.dirname(os.path.realpath(__file__))
+
+def truncateVersion( nlVersion ):
+    '''drop patch-level part of version number, if any'''
+    return '.'.join(nlVersion.split('.')[:-1]) if nlVersion.count('.') > 1 else nlVersion
+
 class neoloadFrameProcessor(batchRunner.frameProcessor):
     '''defines details for installing Neotys Load Generator agent on a worker'''
 
     def installerCmd( self ):
+        truncVersion = truncateVersion( neoloadVersion )
+        scoredVersion = neoloadVersion.replace( '.', '_' )
         if neoloadVersion == '7.10':
-            return 'nlAgent/install_7-10.sh'
+            return 'nlAgent/install_7-10_slim.sh'
         elif neoloadVersion == '7.7':
             return 'nlAgent/install_7-7.sh'
-        else:
+        elif neoloadVersion == '7.6':
             return 'nlAgent/install_7-6.sh'
+        else:
+            return 'nlAgent/install_7-x.sh %s %s %s' % (neoloadVersion, truncVersion, scoredVersion )
 
 def sigtermSignaled():
     return g_.signaled
@@ -59,7 +87,7 @@ def commandInstance( inst, cmd, timeLimit ):
         while time.time() < deadline:
             proc.poll() # sets proc.returncode
             if proc.returncode == None:
-                logger.info( 'waiting for command on instance %s', abbrevIid)
+                logger.debug( 'waiting for command on instance %s', abbrevIid)
             else:
                 if proc.returncode == 0:
                     logger.debug( 'command succeeded on instance %s', abbrevIid )
@@ -83,7 +111,7 @@ def commandInstance( inst, cmd, timeLimit ):
         try:
             proc.wait(timeout=5)
             if proc.returncode:
-                logger.warning( 'ssh return code %d', proc.returncode )
+                logger.debug( 'ssh return code %d', proc.returncode )
         except subprocess.TimeoutExpired:
             logger.warning( 'ssh did not terminate in time' )
         #stderrThr.join()
@@ -101,15 +129,26 @@ def configureAgent( inst, port, timeLimit=500 ):
     iid = inst['instanceId']
     logger.debug( 'would configure agent on instance %s for port %d', iid[0:16], port )
     rc = 1
+    # drop patch-level part of version number, if any
+    truncatedVersion = truncateVersion( neoloadVersion )
     # generate a command to modify agent.properties on the instance
-    configDirPath = '~/neoload%s/conf' % neoloadVersion
+    configDirPath = '~/neoload%s/conf' % truncatedVersion
     if nlWebWanted:
         cmd = "cat %s/nlweb.properties >> %s/agent.properties" % tuple( [configDirPath]*2 )
     else:
         cmd = ":"  # a null command
-    cmd += " && sed -i 's/NCS_LG_PORT/%d/' ~/neoload%s/conf/agent.properties" % (port, neoloadVersion)
-    cmd += " && sed -i 's/NCS_LG_HOST/%s/' ~/neoload%s/conf/agent.properties" % (forwarderHost, neoloadVersion)
-    logger.debug( 'cmd: %s', cmd )
+    cmd += " && sed -i 's/NCS_LG_PORT/%d/' %s/agent.properties" % (port, configDirPath)
+    cmd += " && sed -i 's/NCS_LG_HOST/%s/' %s/agent.properties" % (forwarderHost, configDirPath)
+    if nlWebWanted:
+        # deployment type for nlweb
+        dtype = 'SAAS' if args.nlWebUrl == 'SAAS' else 'ONPREMISE'
+        cmd += " && sed -i 's/NCS_NLWEB_DTYPE/%s/g' %s/agent.properties" % (dtype, configDirPath)
+        # zone for nlweb
+        cmd += " && sed -i 's/NCS_NLWEB_ZONE/%s/g' %s/agent.properties" % (args.nlWebZone, configDirPath)
+        escapedUrl = args.nlWebUrl.replace( '/', '\/' )
+        cmd += " && sed -i 's/NCS_NLWEB_TOKEN/%s/g' %s/agent.properties" % (args.nlWebToken, configDirPath)
+        cmd += " && sed -i 's/NCS_NLWEB_URL/%s/' %s/agent.properties" % (escapedUrl, configDirPath)
+    logger.debug( 'info: %s', cmd )
     rc = commandInstance( inst, cmd, timeLimit=timeLimit )
     return rc
 
@@ -132,6 +171,18 @@ def configureAgents( instances, ports, timeLimit=600 ):
         logger.debug( 'returnCodes: %s', returnCodes )
     return returnCodes
 
+def purgeHostKeys( instanceRecs ):
+    '''try to purgeKnownHosts; warn if any exception'''
+    logger.debug( 'purgeKnownHosts for %d instances', len(instanceRecs) )
+    try:
+        ncs.purgeKnownHosts( instanceRecs )
+    except Exception as exc:
+        logger.warning( 'exception from purgeKnownHosts (%s) %s', type(exc), exc, exc_info=True )
+        return 1
+    else:
+        return 0
+
+
 if __name__ == '__main__':
     # configure logger formatting
     logger = logging.getLogger(__name__)
@@ -140,24 +191,57 @@ if __name__ == '__main__':
     formatter = logging.Formatter(fmt=logFmt, datefmt=logDateFmt )
     logging.basicConfig(format=logFmt, datefmt=logDateFmt)
     #batchRunner.logger.setLevel(logging.DEBUG)  # for more verbosity
+    #startForwarders.logger.setLevel(logging.DEBUG)  # for more verbosity
     logger.setLevel(logging.INFO)
 
     ap = argparse.ArgumentParser( description=__doc__, fromfile_prefix_chars='@',
         formatter_class=argparse.ArgumentDefaultsHelpFormatter )
+    ap.add_argument( '--authToken', help='the NCS authorization token to use (or none, to use NCS_AUTH_TOKEN env var' )
+    ap.add_argument( '--filter', help='json to filter instances for launch',
+        default = '{ "regions": ["asia", "europe", "middle-east", "north-america", "oceania"], "dar": ">=99", "dpr": ">=48", "ram": ">=3800000000", "storage": ">=2000000000" }'
+    )
     ap.add_argument( '--forwarderHost', help='IP addr (or host name) of the forwarder host',
         default='localhost' )
+    ap.add_argument( '--neoloadVersion', default=neoloadVersion, help='version of neoload LG agent' )
+    ap.add_argument( '--nlWeb', type=ncs.boolArg, default=False, help='whether to use NeoLoad Web' )
+    ap.add_argument( '--nlWebToken', help='a token for authorized access to a neoload web server' )
+    ap.add_argument( '--nlWebUrl', help='the URL of a neoload web server to query' )
+    ap.add_argument( '--nlWebZone', help='the neoload zone that the agents should belong to',
+        default='defaultzone' )
     ap.add_argument( '--nWorkers', type=int, help='the number of agents to launch',
         default=10 )
+    ap.add_argument( '--outDataDir', required=False, help='a path to the output data dir for this run' )
+    ap.add_argument( '--portRangeStart', type=int, default=7100,
+        help='the beginning of the range of port numbers to forward' )
+    ap.add_argument( '--supportedVersions', action='store_true', help='to list supported versions and exit' )
     args = ap.parse_args()
 
-    supportedVersions = ['7.6', '7.7', '7.10']
+    supportedVersions = ['7.6', '7.7', '7.10', '7.10.0', '7.10.1', '7.11.0']
+    if args.supportedVersions:
+        print( json.dumps( supportedVersions ) )
+        sys.exit( 0 )
+    neoloadVersion =  args.neoloadVersion
     if neoloadVersion not in supportedVersions:
         logger.error( 'version "%s" is not suppoprted; supported versions are %s',
             neoloadVersion, sorted( supportedVersions ) )
         sys.exit( 1 )
 
-    dateTimeTag = datetime.datetime.now().strftime( '%Y-%m-%d_%H%M%S' )
-    outDataDir = 'data/neoload_' + dateTimeTag
+    nlWebWanted = args.nlWeb
+
+    if nlWebWanted:
+        # make sure all the necessary nlWeb args were passed in non-empty
+        if not args.nlWebToken:
+            logger.error( 'please pass a non-empty --nlWebToken if you want to use NeoLoad Web')
+        if not args.nlWebUrl:
+            logger.error( 'please pass a non-empty --nlWebUrl if you want to use NeoLoad Web')
+        if not (args.nlWebUrl and args.nlWebUrl):
+            sys.exit( 1 )
+
+
+    outDataDir = args.outDataDir
+    if not outDataDir:
+        dateTimeTag = datetime.datetime.now().strftime( '%Y-%m-%d_%H%M%S' )
+        outDataDir = 'data/neoload_' + dateTimeTag
 
     # you may set forwarderHost manually here, to override auto-detect
     forwarderHost = args.forwarderHost
@@ -170,8 +254,24 @@ if __name__ == '__main__':
         logger.error( 'forwarderHost not set')
         exit(1)
     
-    authToken = os.getenv('NCS_AUTH_TOKEN') or 'YourAuthTokenHere'
-    instTimeLimit = 13*60 if neoloadVersion in ['7.10'] else 30*60
+    authToken = args.authToken or os.getenv('NCS_AUTH_TOKEN')
+    instTimeLimit = 11*60  #  if neoloadVersion in ['7.10'] else 30*60
+
+    nlAgentDirName = 'nlAgent'
+    if not os.path.isdir( nlAgentDirName ):
+        if os.path.exists( nlAgentDirName ) and not os.path.islink( nlAgentDirName ):
+            logger.error( 'you have an nlAgent that is neither a dir nor a symlink')
+            sys.exit( 1 )
+        targetPath = os.path.join( scriptDirPath(), nlAgentDirName )
+        if not os.path.isdir( targetPath ):
+            logger.error( 'nlAgent dir not found in %s', scriptDirPath() )
+            sys.exit( 1 )
+        try:
+            os.symlink( targetPath, nlAgentDirName, target_is_directory=True )
+        except Exception as exc:
+            logger.error( 'could not create symlink for nlAgent (%s) %s', type(exc), exc)
+            sys.exit( 1 )
+    logger.debug( 'nlAgent contents: %s', os.listdir(nlAgentDirName) )
 
     if nlWebWanted and not os.path.isfile( 'nlAgent/nlweb.properties'):
         logger.error( 'the file nlAgent/nlweb.properties was not found')
@@ -187,16 +287,33 @@ if __name__ == '__main__':
             encryptFiles=False,
             timeLimit = 60*60,
             instTimeLimit = instTimeLimit,
-            filter = '{ "dar": "==100", "regions": ["usa"], "dpr": ">=48","ram:":">=5800000000","app-version": ">=2.1.14"}',
+            filter = args.filter,
             outDataDir = outDataDir,
             nWorkers = args.nWorkers
         )
         if rc == 0:
-            portRangeStart=7100
+            # get iids of instances successfully installed
+            recruiterJlogFilePath = os.path.join( outDataDir, 'recruitInstances.jlog' )
+            recruitedIids = []
+            if os.path.isfile( recruiterJlogFilePath ):
+                recruiterResults = readJLog( recruiterJlogFilePath )
+                if not recruiterResults:
+                    logger.warning( 'no entries in %s', recruiterJlogFilePath )
+                for result in recruiterResults:
+                    if 'timeout' in result:
+                        logger.debug( 'recruiter timeout: %s', result )
+                    elif 'returncode' in result:
+                        if result['returncode'] != 0:
+                            logger.debug( 'recruiter result: %s', result )
+                        else:
+                            recruitedIids.append( result.get( 'instanceId' ) )
+            recruitedIids = set( recruitedIids )
+            logger.debug( '%d recruitedIids: %s', len(recruitedIids), recruitedIids )
+
+            portRangeStart=args.portRangeStart
             launchedJsonFilePath = outDataDir +'/recruitLaunched.json'
             launchedInstances = []
             # get details of launched instances from the json file
-            #TODO should get list of instances with good install, rather than all started instances
             with open( launchedJsonFilePath, 'r') as jsonInFile:
                 try:
                     launchedInstances = json.load(jsonInFile)  # an array
@@ -204,17 +321,21 @@ if __name__ == '__main__':
                     logger.warning( 'could not load json (%s) %s', type(exc), exc )
                     sys.exit( 2 )
             launchedIids = [inst['instanceId'] for inst in launchedInstances ]
-            startedInstances = [inst for inst in launchedInstances if inst['state'] == 'started' ]
+            #startedInstances = [inst for inst in launchedInstances if inst['state'] == 'started' ]
             #logger.info( '%d instances were launched', len(startedInstances) )
 
-            if neoloadVersion == '7.10':
-                agentLogFilePath = '/root/.neotys/neoload/v7.10/logs/agent.log'
-                starterCmd = 'cd ~/neoload7.10/ && /usr/bin/java -Xms512m -Xmx512m -Dvertx.disableDnsResolver=true -classpath $HOME/neoload7.10/.install4j/i4jruntime.jar:$HOME/neoload7.10/.install4j/launchera03c11da.jar:$HOME/neoload7.10/bin/*:$HOME/neoload7.10/lib/crypto/*:$HOME/neoload7.10/lib/*:$HOME/neoload7.10/lib/jdbcDrivers/*:$HOME/neoload7.10/lib/plugins/ext/* install4j.com.neotys.nl.agent.launcher.AgentLauncher_LoadGeneratorAgent start &'
-            elif neoloadVersion == '7.7':
-                agentLogFilePath = '/root/.neotys/neoload/v7.7/logs/agent.log'
-                starterCmd = 'cd ~/neoload7.7/ && /usr/bin/java -Xmx512m -Dvertx.disableDnsResolver=true -classpath $HOME/neoload7.7/.install4j/i4jruntime.jar:$HOME/neoload7.7/.install4j/launchera03c11da.jar:$HOME/neoload7.7/bin/*:$HOME/neoload7.7/lib/crypto/*:$HOME/neoload7.7/lib/*:$HOME/neoload7.7/lib/jdbcDrivers/*:$HOME/neoload7.7/lib/plugins/ext/* install4j.com.neotys.nl.agent.launcher.AgentLauncher_LoadGeneratorAgent start &'
-            else:
-                agentLogFilePath = '/root/.neotys/neoload/v7.6/logs/agent.log'
+            startedInstances = [inst for inst in launchedInstances if inst['instanceId'] in recruitedIids ]
+
+            #COULD check memory and available ports here
+
+            truncVersion = truncateVersion( neoloadVersion )
+            agentLogFilePath = '/root/.neotys/neoload/v%s/logs/agent.log' % truncVersion
+            starterCmd = 'cd ~/neoload7.xx/ && /usr/bin/java -Xms50m -Xmx100m -Dvertx.disableDnsResolver=true -classpath $HOME/neoload7.xx/.install4j/i4jruntime.jar:$HOME/neoload7.xx/.install4j/launchera03c11da.jar:$HOME/neoload7.xx/bin/*:$HOME/neoload7.xx/lib/crypto/*:$HOME/neoload7.xx/lib/*:$HOME/neoload7.xx/lib/jdbcDrivers/*:$HOME/neoload7.xx/lib/plugins/ext/* install4j.com.neotys.nl.agent.launcher.AgentLauncher_LoadGeneratorAgent start & sleep 30 && free --mega 1>&2'
+            starterCmd = starterCmd.replace( 'neoload7.xx', 'neoload'+truncVersion )
+
+            if neoloadVersion == '7.7':
+                starterCmd = 'cd ~/neoload7.7/ && /usr/bin/java -Xms50m -Xmx100m -Dvertx.disableDnsResolver=true -classpath $HOME/neoload7.7/.install4j/i4jruntime.jar:$HOME/neoload7.7/.install4j/launchera03c11da.jar:$HOME/neoload7.7/bin/*:$HOME/neoload7.7/lib/crypto/*:$HOME/neoload7.7/lib/*:$HOME/neoload7.7/lib/jdbcDrivers/*:$HOME/neoload7.7/lib/plugins/ext/* install4j.com.neotys.nl.agent.launcher.AgentLauncher_LoadGeneratorAgent start & sleep 30'
+            elif neoloadVersion == '7.6':
                 starterCmd = 'cd ~/neoload7.6/ && /usr/bin/java -Dneotys.vista.headless=true -Xmx512m -Dvertx.disableDnsResolver=true -classpath $HOME/neoload7.6/.install4j/i4jruntime.jar:$HOME/neoload7.6/.install4j/launcherc0a362f9.jar:$HOME/neoload7.6/bin/*:$HOME/neoload7.6/lib/crypto/*:$HOME/neoload7.6/lib/*:$HOME/neoload7.6/lib/jdbcDrivers/*:$HOME/neoload7.6/lib/plugins/ext/* install4j.com.neotys.nl.agent.launcher.AgentLauncher_LoadGeneratorAgentService start &'
 
             configuredInstances = []
@@ -225,6 +346,7 @@ if __name__ == '__main__':
                 for index, inst in enumerate( startedInstances ):
                     iid = inst['instanceId']
                     portMap[iid] = index + portRangeStart
+                logger.info( 'configuring agents')
                 returnCodes = configureAgents( startedInstances, ports, timeLimit=600 )
                 for index, code in enumerate( returnCodes ):
                     if code==0:
@@ -247,6 +369,8 @@ if __name__ == '__main__':
                     goodIids.append( status['instanceId'])
                 else:
                     logger.warning( 'could not start agent on %s', status['instanceId'][0:8] )
+            #COULD check bound ports again here
+            #COULD download logs from all installed instances rather than just good-started instances
             goodInstances = [inst for inst in startedInstances if inst['instanceId'] in goodIids ]
             if goodInstances:
                 time.sleep( 60 )
@@ -279,15 +403,14 @@ if __name__ == '__main__':
                         logger.warning( 'could not download log from %s', status['instanceId'][0:8] )
                 goodInstances = [inst for inst in goodInstances if inst['instanceId'] in goodIids ]
                 with open( outDataDir + '/startedAgents.json','w' ) as outFile:
-                    json.dump( goodInstances, outFile )
+                    json.dump( goodInstances, outFile, indent=2 )
 
                 # plot map of workers
                 if os.path.isfile( outDataDir +'/startedAgents.json' ):
-                    rc2 = subprocess.call( ['./plotAgentMap.py', '--dataDirPath', outDataDir],
+                    rc2 = subprocess.call( [scriptDirPath()+'/plotAgentMap.py', '--dataDirPath', outDataDir],
                         stdout=subprocess.DEVNULL )
                     if rc2:
                         logger.warning( 'plotAgentMap exited with returnCode %d', rc2 )
-
                 # start the ssh port-forwarding
                 logger.info( 'would forward ports for %d instances', len(goodInstances) )
                 forwarders = startForwarders.startForwarders( goodInstances,
@@ -303,11 +426,14 @@ if __name__ == '__main__':
                 forwardedIids = [inst['instanceId'] for inst in goodInstances ]
                 unusableIids = list( set(launchedIids) - set( forwardedIids) )
                 if unusableIids:
-                    logger.info( 'terminating %d unusable instances', len(unusableIids) )
+                    logger.debug( 'terminating %d unusable instances', len(unusableIids) )
                     ncs.terminateInstances( authToken, unusableIids )
+                    unusableInstances = [inst for inst in launchedInstances \
+                        if inst['instanceId'] in unusableIids]
+                    purgeHostKeys( unusableInstances )
             if launchedInstances:
-                print( 'when you want to terminate these instances, use %s terminateAgents.py "%s"'
-                    % (sys.executable, outDataDir))
+                print( 'when you want to terminate these instances, use %s %s "%s"'
+                    % (sys.executable, scriptDirPath()+'/terminateAgents.py', outDataDir))
         sys.exit( rc )
     except KeyboardInterrupt:
         logger.warning( 'an interuption occurred')
