@@ -523,12 +523,12 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
         badInstances = []
         if startedInstances:
             # avoid calling checkInstanceClocks on windows because stdCommandInstance runs very slowly there
-            if sys.platform.startswith( 'win32' ):
+            if True or sys.platform.startswith( 'win32' ):
                 returnCodes = []
             else:
-                returnCodes = checkInstanceClocks( startedInstances, timeLimit=120 )
+                returnCodes = checkInstanceClocks( startedInstances, timeLimit=180 )
             if not any( returnCodes ):
-                logger.info( 'checkInstanceClocks found all good' )
+                logger.debug( 'checkInstanceClocks found all good' )
                 goodInstances = startedInstances
             else:
                 logger.warning( 'checkInstanceClocks returned %s', returnCodes )
@@ -628,6 +628,8 @@ def recruitInstances( nWorkersWanted, launchedJsonFilePath, launchWanted, result
                 jobId = launchedInstances[0].get('job')
                 if jobId:
                     ncs.terminateJobInstances( args.authToken, jobId )
+                # else (never happens) could terminateInstances using iids
+                purgeHostKeys( launchedInstances )
         # delete sshClientKey only if we just uploaded it
         if sshClientKeyName != args.sshClientKeyName:
             logger.debug( 'deleting sshClientKey %s', sshClientKeyName)
@@ -648,7 +650,8 @@ def rsyncFromRemote1( srcFileName, destFilePath, inst, timeLimit ):
     user = sshSpecs['user']
 
     destFilePathFull = os.path.realpath(os.path.abspath( destFilePath ))
-    cmd = [ 'rsync', '-a', '-e', 'ssh -p ' + str(port), user+'@'+host+':~/'+srcFileName,
+    timeLimitMinutes = max( 1, int( timeLimit/60 ) )
+    cmd = [ 'rsync', '--time-limit=%d' % timeLimitMinutes, '-a', '-e', 'ssh -p ' + str(port), user+'@'+host+':~/'+srcFileName,
         destFilePathFull+'/'
     ]
     logger.info( 'retrieving from %s', inst['instanceId'] )
@@ -888,8 +891,8 @@ def renderFramesOnInstance( inst ):
         if time.time() >= g_.deadline:
             logger.warning( 'exiting thread because global deadline has passed' )
             break
-        if nFailures >= 3:
-            logger.warning( 'exiting thread because instance has encountered %d failures', nFailures )
+        if nFailures >= 2:
+            logger.warning( 'exiting thread because instance %s has encountered %d failures', abbrevIid, nFailures )
             logOperation( 'terminateFailedWorker', iid, '<master>')
             terminateInstances( args.authToken, [iid] )
             purgeHostKeys( [inst] )
@@ -1016,6 +1019,7 @@ def renderFramesOnInstance( inst ):
                 logFrameState( frameNum, 'retrieveFailed', iid, returnCode )
                 logger.warning( 'retrieveFailed with rc %d for frame %d on %s', returnCode, frameNum, iid )
                 frameDetails[ 'progress' ] = 0
+                time.sleep( 10 )
             saveProgress()
         if returnCode:
             nFailures += 1
@@ -1381,10 +1385,12 @@ def runBatch( **kwargs ):
 
     if not args.nWorkers:
         # regular case, where we pick a suitably large number to launch, based on # of frames
-        nAvail = round( ncs.getAvailableDeviceCount( args.authToken, filtersJson=args.filter ) * .9 )
+        nAvail = ncs.getAvailableDeviceCount( args.authToken, filtersJson=args.filter )
         logger.debug( 'args.filter: %s', args.filter )
         logger.info( '%d filtered devices available', nAvail )
         nFrames = len( range(args.startFrame, args.endFrame+1, args.frameStep ) )
+        # cap the number at significantly less than all of them
+        nAvail = round( nAvail * .5 )
         nToRecruit = min( nAvail, round( nFrames * g_.autoscaleInit ) )
         logger.debug( 'recruiting up to %d instances', nToRecruit )
     elif args.nWorkers > 0:
